@@ -1,5 +1,5 @@
 #
-# Copyright (c) 2019 cPanel, L.L.C.
+# Copyright (c) 2015 cPanel, Inc.
 # All rights reserved.
 # http://cpanel.net/
 #
@@ -17,9 +17,7 @@ use LWP::UserAgent ();
 use JSON        ();
 use URI::Encode ();
 
-use OpenStack::Client::Response ();
-
-our $VERSION = '1.0006_0001';
+our $VERSION = '1.0001';
 
 =encoding utf8
 
@@ -30,14 +28,11 @@ OpenStack::Client - A cute little client to OpenStack services
 =head1 SYNOPSIS
 
     #
-    # First, connect to an API endpoint via the Keystone
-    # authorization service
+    # First, connect to an API endpoint via the Keystone authorization service
     #
     use OpenStack::Client::Auth ();
 
-    my $endpoint = 'http://openstack.foo.bar:5000/v2.0';
-
-    my $auth = OpenStack::Client::Auth->new($endpoint,
+    my $auth = OpenStack::Client::Auth->new('http://openstack.foo.bar:5000/v2.0',
         'tenant'   => $ENV{'OS_TENANT_NAME'},
         'username' => $ENV{'OS_USERNAME'},
         'password' => $ENV{'OS_PASSWORD'}
@@ -54,9 +49,7 @@ OpenStack::Client - A cute little client to OpenStack services
     #
     use OpenStack::Client ();
 
-    my $endpoint = 'http://glance.foo.bar:9292';
-
-    my $glance = OpenStack::Client->new($endpoint,
+    my $glance = OpenStack::Client->new('http://glance.foo.bar:9292',
         'token' => {
             'id' => 'foo'
         }
@@ -104,11 +97,10 @@ A token obtained from a L<OpenStack::Client::Auth> object.
 sub new ($%) {
     my ($class, $endpoint, %opts) = @_;
 
-    die 'No API endpoint provided' unless $endpoint;
+    die('No API endpoint provided') unless $endpoint;
 
-    $opts{'package_ua'}       ||= 'LWP::UserAgent';
-    $opts{'package_request'}  ||= 'HTTP::Request';
-    $opts{'package_response'} ||= 'OpenStack::Client::Response';
+    $opts{'package_ua'}      ||= 'LWP::UserAgent';
+    $opts{'package_request'} ||= 'HTTP::Request';
 
     my $ua = $opts{'package_ua'}->new(
         'ssl_opts' => {
@@ -117,12 +109,11 @@ sub new ($%) {
     );
 
     return bless {
-        'package_ua'       => $opts{'package_ua'},
-        'package_request'  => $opts{'package_request'},
-        'package_response' => $opts{'package_response'},
-        'ua'               => $ua,
-        'endpoint'         => $endpoint,
-        'token'            => $opts{'token'}
+        'package_ua'      => $opts{'package_ua'},
+        'package_request' => $opts{'package_request'},
+        'ua'              => $ua,
+        'endpoint'        => $endpoint,
+        'token'           => $opts{'token'}
     }, $class;
 }
 
@@ -173,109 +164,12 @@ sub uri ($$) {
 
 =over
 
-=cut
+=item C<$client-E<gt>call(I<$method>, I<$path>, I<$body>)>
 
-sub request {
-    my ($self, %args) = @_;
-
-    $args{'headers'} ||= {};
-
-    my $request = $self->{'package_request'}->new(
-        $args{'method'} => $self->uri($args{'path'})
-    );
-
-    my @headers = $self->_get_headers_list($args{'headers'});
-
-    my $count = scalar @headers;
-
-    for (my $i=0; $i<$count; $i+=2) {
-        my $name  = $headers[$i];
-        my $value = $headers[$i+1];
-
-        $request->header($name => $value);
-    }
-
-    if (defined $args{'body'}) {
-        #
-        # Allow the request body to be supplied by a subroutine reference
-        # which, when called, will supply a chunk of data returned as per the
-        # behavior of LWP::UserAgent.  This is useful for uploading arbitrary
-        # amounts of data in a request body.
-        #
-        if (ref($args{'body'}) =~ /CODE/) {
-            $request->content($args{'body'});
-        } else {
-            $request->content(JSON::encode_json($args{'body'}));
-        }
-    }
-
-    return bless $self->{'ua'}->request($request,
-        defined $args{'handler'}? $args{'handler'}: ()),
-        $self->{'package_response'};
-}
-
-=item C<$client-E<gt>call(I<$args>)>
-
-Perform a call to the service endpoint using named arguments in the hash.  The
-following arguments are required:
-
-=over
-
-=item C<method> - Request method
-
-=item C<path> - Resource path
-
-=back
-
-The following arguments are optional:
-
-=over
-
-=item C<headers> - Request headers
-
-Headers are case I<insensitive>; if duplicate header values are declared under
-different cases, it is undefined which headers shall take precedence.  The
-following headers are sent by default:
-
-=over
-
-=item Accept
-
-Defaults to C<application/json, text/plain>.
-
-=item Accept-Encoding
-
-Defaults to C<identity, gzip, deflate, compress>.
-
-=item Content-Type
-
-Defaults to C<application/json>, although some API calls (e.g., a PATCH)
-expect a different type; the case of an image update, the expected
-type is C<application/openstack-images-v2.1-json-patch> or some version
-thereof.
-
-=back
-
-Except for C<X-Auth-Token>, any additional token will be added to the request.
-
-=item C<body> - Request body
-
-This may be a scalar reference to a data structure to be encoded to JSON, or a
-CODE reference to a subroutine which, when called, will return a chunk of data
-to be supplied to the API endpoint; the stream is ended when the supplied
-subroutine returns an empty string or undef.
-
-=item C<handler> - Response body handler function
-
-When specified, this function will be called with two arguments; the first
-argument is a scalar value containing a chunk of data in the response body, and
-the second is a scalar reference to a L<HTTP::Response> object representing the
-current response.  This is useful for retrieving very large resources without
-having to store the entire response body in memory at once for parsing.
-
-=back
-
-All forms of this method may return the following:
+Perform a call to the service endpoint using the HTTP method I<$method>,
+accessing the resource I<$path> (relative to the absolute endpoint URI), passing
+an arbitrary value in I<$body> that is to be encoded to JSON as a request
+body.  This method may return the following:
 
 =over
 
@@ -286,127 +180,59 @@ All forms of this method may return the following:
 =back
 
 In exceptional conditions (such as when the service returns a 4xx or 5xx HTTP
-response), the client will die with the raw text response from the HTTP
+response), the client will C<die()> with the raw text response from the HTTP
 service, indicating the nature of the service-side failure to service the
 current call.
 
-=item C<$client-E<gt>call(I<$method>, I<$path>, I<$body>)>
-
-Perform a call to the service endpoint using the HTTP method I<$method>,
-accessing the resource I<$path> (relative to the absolute endpoint URI),
-passing an arbitrary value in I<$body>.
-
-=item C<$client-E<gt>call(I<$method>, I<$headers>, I<$path>, I<$body>)>
-
-Perform a call to the service endpoint using the HTTP method I<$method>,
-accessing the resource I<$path> (relative to the absolute endpoint URI),
-specifying the headers in I<$headers>, passing an arbitrary value in I<$body>.
-
-=back
-
-=head1 EXAMPLES
-
-The following shows how one may update image metadata using the PATCH method
-supported by version 2 of the Image API.  C<@image_updates> is an array of hash
-references of the structure defined by the PATCH RFC (6902) governing
-"JavaScript Object Notation (JSON) Patch"; i.e., operations consisting of
-C<add>, C<replace>, or C<delete>.
-
-    my $headers = {
-        'Content-Type' => 'application/openstack-images-v2.1-json-patch'
-    };
-
-    my $response = $glance->call({
-        'method'  => 'PATCH',
-        'headers' => $headers,
-        'path'    => qq[/v2/images/$image->{id}],
-        'body'    => \@image_updates
-    );
-
 =cut
 
-sub call {
-    my $self = shift;
+sub call ($$$$) {
+    my ($self, $method, $path, $body) = @_;
 
-    if (scalar @_ == 4) {
-        return $self->call({
-            'method'  => $_[0],
-            'headers' => $_[1],
-            'path'    => $_[2],
-            'body'    => $_[3]
-        });
-    } elsif (scalar @_ == 3) {
-        return $self->call({
-            'method'  => $_[0],
-            'headers' => {},
-            'path'    => $_[1],
-            'body'    => $_[2],
-        });
-    } elsif (scalar @_ == 2) {
-        return $self->call({
-            'method'  => $_[0],
-            'headers' => {},
-            'path'    => $_[1],
-            'body'    => undef
-        });
-    } elsif (scalar @_ != 1) {
-        die "Invalid number of arguments: @_";
-    }
+    my $request = $self->{'package_request'}->new(
+        $method => $self->uri($path)
+    );
 
-    my ($args) = @_;
-
-    return $self->request(%{$args})->decode_json;
-}
-
-sub _lc_merge {
-    my ($a, $b, %opts) = @_;
-
-    my %lc_keys_a = map {
-        lc $_ => $_
-    } keys %{$a};
-
-    foreach my $key_b (keys %{$b}) {
-        my $key_a = $lc_keys_a{lc $key_b};
-
-        if (!defined($key_a)) {
-            $a->{$key_b} = $b->{$key_b};
-        } elsif (exists $a->{$key_a} && $opts{'replace'}) {
-            $a->{$key_a} = $b->{$key_b};
-        }
-    }
-
-    return;
-}
-
-#
-# Internal method for call() to process headers; returns a list of header name
-# and value pairs
-#
-sub _get_headers_list {
-    my ($self, $headers) = @_;
-
-    my %DEFAULTS = (
+    my @headers = (
         'Accept'          => 'application/json, text/plain',
         'Accept-Encoding' => 'identity, gzip, deflate, compress',
         'Content-Type'    => 'application/json'
     );
 
-    #
-    # The client should be not adding X-Auth-Token explicitly, so force it to
-    # the one received during authentication
-    #
-    my %OVERRIDES = (
-        'X-Auth-Token' => $self->token
-    );
+    push @headers, (
+        'X-Auth-Token' => $self->{'token'}->{'id'}
+    ) if defined $self->{'token'}->{'id'};
 
-    my %new_headers = %{$headers};
+    my $count = scalar @headers;
 
-    _lc_merge(\%new_headers, \%DEFAULTS);
-    _lc_merge(\%new_headers, \%OVERRIDES, 'replace' => 1);
+    for (my $i=0; $i<$count; $i+=2) {
+        my $name  = $headers[$i];
+        my $value = $headers[$i+1];
 
-    return %new_headers;
+        $request->header($name => $value);
+    }
+
+    $request->content(JSON::encode_json($body)) if defined $body;
+
+    my $response = $self->{'ua'}->request($request);
+    my $type     = $response->header('Content-Type');
+    my $content  = $response->decoded_content;
+
+    if ($response->code =~ /^[45]\d{2}$/) {
+        $content ||= "@{[$response->code]} Unknown error";
+
+        die($content);
+    }
+
+    if (lc($type) =~ qr{^application/json}i && defined $content && length $content) {
+        return JSON::decode_json($content);
+    } else {
+        return $content;
+    }
 }
 
+=back
+
 =head1 FETCHING REMOTE RESOURCES
 
 =over
@@ -440,7 +266,7 @@ sub get ($$%) {
         # subsequent values with a & rather than ?.
         #
         if ($path =~ /\?/) {
-            $path .= "&$params";
+            $path .= "&$params"
         } else {
             $path .= "?$params";
         }
@@ -471,7 +297,7 @@ sub each ($$@) {
     } elsif (scalar @args == 1) {
         ($callback) = @args;
     } else {
-        die 'Invalid number of arguments';
+        die('Invalid number of arguments');
     }
 
     while (defined $path) {
@@ -508,15 +334,14 @@ sub every ($$$@) {
     } elsif (scalar @args == 1) {
         ($callback) = @args;
     } else {
-        die 'Invalid number of arguments';
+        die('Invalid number of arguments');
     }
 
     while (defined $path) {
         my $result = $self->get($path, %{$opts});
 
         unless (defined $result->{$attribute}) {
-            my $keys = join( ', ', sort keys %$result );
-            die "Response from $path does not contain attribute '$attribute', possible options are " . $keys;
+            die("Response from $path does not contain attribute '$attribute'");
         }
 
         foreach my $item (@{$result->{$attribute}}) {
@@ -619,19 +444,11 @@ The OpenStack Keystone authentication and authorization interface
 
 =head1 AUTHOR
 
-Written by Alexandra Hrefna Maheu <xan@cpanel.net>
-
-=head1 CONTRIBUTORS
-
-=over
-
-=item Brett Estrade <brett@cpanel.net>
-
-=back
+Written by Alexandra Hrefna Hilmisdóttir <xan@cpanel.net>
 
 =head1 COPYRIGHT
 
-Copyright (c) 2019 cPanel, L.L.C.  Released under the terms of the MIT license.
+Copyright (c) 2015 cPanel, Inc.  Released under the terms of the MIT license.
 See LICENSE for further details.
 
 =cut
